__author__ = 'saeedamen' # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

"""
DataConstants

Has various constants required for the findatapy project. These have been defined as static variables.

"""

import os
import keyring

def key_store(service_name):
    key = None

    # this will fail on some cloud notebook platforms so put in try/except loop
    try:
        key = keyring.get_password(service_name, os.getlogin())
    except:
        pass

    # set the keys by running set_api_keys.py file!

    # if key is None:
    #    key = input("Please enter the %s API key: " % service_name)
    #
    #    keyring.set_password(service_name, os.getlogin(), key)

    return key

class DataConstants(object):

    ###### SHOULD AUTODETECT FOLDER
    root_folder = os.path.dirname(os.path.dirname(os.path.abspath(__file__))).replace('\\', '/') + "/"
    temp_folder = root_folder + "temp"

    ###### FOR FUTURE VERSIONS (which include caching)
    # folders for holding market data
    folder_historic_CSV = "x:/"
    folder_time_series_data = "x:/"

    ###### FOR DATABASE
    db_server = '127.0.0.1'
    db_port = '27017'

    ###### FOR TEMPORARY IN-MEMRORY CACHE (Redis)
    db_cache_server = '127.0.0.1'
    db_cache_port = '6379'
    write_cache_engine = 'redis'  # 'redis' or 'no_cache' means we don't use cache

    use_cache_compression = True

    ###### FOR ALIAS TICKERS
    # config file for time series categories
    config_root_folder = root_folder

    time_series_categories_fields = \
        root_folder + "conf/time_series_categories_fields.csv"

    # we can have multiple tickers files (separated by ";")
    time_series_tickers_list = root_folder + "conf/time_series_tickers_list.csv;" + \
                               root_folder + "conf/fx_vol_tickers.csv;" + \
                               root_folder + "conf/fx_forwards_tickers.csv;" + \
                               root_folder + "conf/base_depos_tickers_list.csv"


    time_series_fields_list = root_folder + "conf/time_series_fields_list.csv"

    # config file for long term econ data
    all_econ_tickers = root_folder + "conf/all_econ_tickers.csv"
    econ_country_codes = root_folder + "conf/econ_country_codes.csv"
    econ_country_groups = root_folder + "conf/econ_country_groups.csv"

    holidays_parquet_table = config_root_folder + '/conf/holidays_table.parquet'

    # for events filtering
    events_category = 'events'
    events_category_dt = 'events_dt'

    ###### FOR CURRENT VERSION

    # which marketdatagenerator type to use?
    # note - marketdatagenerator currently implemented
    #        cachedmarketdatagenerator is only for proprietary version at present
    default_market_data_generator = "marketdatagenerator"

    # in Python threading does not offer true parallisation, but can be useful when downloading data, because
    # a lot of the time is spend waiting on data, multiprocessing library addresses this problem by spawning new Python
    # instances, but this has greater overhead (maybe more advisable when downloading very long time series)

    # "thread" or "multiprocessing" (experimental!) library to use when downloading data
    market_thread_technique = "thread"

    multiprocessing_library = 'multiprocess' # 'multiprocessing_on_dill' or 'multiprocess' or 'multiprocessing'

    # how many threads to use for loading external data (don't do too many on slow machines!)
    # also some data sources will complain if you start too many parallel threads to call data!
    # for some data providers might get better performance from 1 thread only!
    market_thread_no = {             'quandl'      : 4,
                                     'bloomberg'   : 4,
                                     'yahoo'       : 1, # yfinance already threads requests, so don't do it twice!
                                     'other'       : 4,
                                     'dukascopy'   : 8,
                                     'fxcm'        : 4}

    # seconds for timeout
    timeout_downloader = {'dukascopy' : 20}

    dukascopy_multithreading = True

    # we can override the thread count and drop back to single thread for certain market data downloads, as can have issues with
    # quite large daily datasets from Bloomberg (and other data vendors) when doing multi-threading, so can override and use
    # single threading on these (and also split into several chunks)
    #
    override_multi_threading_for_categories = []

    # log config file
    logging_conf = root_folder + "conf/logging.conf"

    # Bloomberg settings
    bbg_server = "localhost"       # needs changing if you use Bloomberg Server API
    bbg_server_port = 8194

    # Dukascopy settings
    dukascopy_base_url = "http://www.dukascopy.com/datafeed/"
    dukascopy_write_temp_tick_disk = False

    # FXCM settings
    fxcm_base_url = 'https://tickdata.fxcorporate.com/'
    fxcm_write_temp_tick_disk = False

    # Quandl settings
    quandl_api_key = key_store("Quandl")

    # Alpha Vantage settings
    alpha_vantage_api_key = key_store("AlphaVantage")

    # FXCM API (contact FXCM to get this)
    fxcm_api_key = "x"

    # Eikon settings
    eikon_api_key = key_store("Eikon")

    # Twitter settings (you need to set these up on Twitter)
    TWITTER_APP_KEY             = key_store("Twitter App Key")
    TWITTER_APP_SECRET          = key_store("Twitter App Secret")
    TWITTER_OAUTH_TOKEN	        = key_store("Twitter OAUTH token")
    TWITTER_OAUTH_TOKEN_SECRET	= key_store("Twitter OAUTH token Secret")

    # FRED (Federal Reserve of St Louis data) settings
    fred_api_key = key_store("FRED")

    # Default download for FX vol surfaces etc.
    # types of quotation on vol surface
    # ATM, 25d riskies, 10d riskies, 25d strangles/butterflies, 10d strangles/butterflies
    fx_vol_part = ["V", "25R", "10R", "25B", "10B"]

    # Deltas quoted, eg 10d and 25d
    fx_vol_delta = [10, 25]

    # All the tenors on our vol surface
    fx_vol_tenor = ["ON", "1W", "2W", "3W", "1M", "2M", "3M", "4M", "6M", "9M", "1Y", "2Y", "3Y", "5Y"]

    # Which base depo currencies are available?
    base_depos_currencies = ['EUR', 'GBP', 'AUD', 'NZD', 'USD', 'CAD', 'CHF', 'NOK', 'SEK', 'JPY']

    # Tenors available for base depos
    base_depos_tenor = ["ON", "TN", "SN", "1W", "2W", "3W", "1M", "2M", "3M", "4M", "6M", "9M", "1Y", "2Y", "3Y", "5Y"]

    ### FX forwards total return index construction
    # All the tenors on our forwards
    fx_forwards_tenor = ["ON", "TN", "SN", "1W", "2W", "3W", "1M", "2M", "3M", "4M", "6M", "9M", "1Y", "2Y", "3Y", "5Y"]

    # overwrite field variables with those listed in DataCred
    def __init__(self):
        try:
            from findatapy.util.datacred import DataCred
            cred_keys = DataCred.__dict__.keys()

            for k in DataConstants.__dict__.keys():
                if k in cred_keys and '__' not in k:
                    setattr(DataConstants, k, getattr(DataCred, k))
        except:
            pass

    @staticmethod
    def reset_api_key(service_name, api_key):
        keyring.set_password(service_name, os.getlogin(), api_key)



__author__ = 'saeedamen' # Saeed Amen

#
# Copyright 2016-2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#


if __name__ == '__main__':
    ###### below line CRUCIAL when running Windows, otherwise multiprocessing doesn't work! (not necessary on Linux)
    from findatapy.util import SwimPool; SwimPool()

    from findatapy.market import Market, MarketDataRequest, MarketDataGenerator

    market = Market(market_data_generator=MarketDataGenerator())

    # download event data from Bloomberg
    # we have to use the special category "events" keyword for economic data events
    # so findatapy can correctly identify them (given the underlying Bloomberg API calls are all different, however,
    # this will appear transparent to the user)
    market_data_request = MarketDataRequest(
                    start_date = "year",
                    category = "events",
                    data_source = 'bloomberg',                                                              # use Bloomberg as data source
                    tickers = ['FOMC', 'NFP'],
                    fields = ['release-date-time-full', 'release-dt', 'actual-release'],                    # which fields to download
                    vendor_tickers = ['FDTR Index', 'NFP TCH Index'],                                       # ticker (Bloomberg)
                    vendor_fields = ['ECO_FUTURE_RELEASE_DATE_LIST', 'ECO_RELEASE_DT', 'ACTUAL_RELEASE'])   # which Bloomberg fields to download

    df = market.fetch_market(market_data_request)

    print(df)

    # we also have a few events defined in our configuation file
    # those tickers/fields which are predefined this way are easier to download
    # note how we don't have to use the vendor_tickers and vendor_fields for examples
    market_data_request = MarketDataRequest(
                    start_date = "year",
                    category = "events",
                    data_source = 'bloomberg',                      # use Bloomberg as data source
                    tickers = ['USD-US Employees on Nonfarm Payrolls Total MoM Net Change SA'],
                    fields = ['release-date-time-full', 'release-dt', 'actual-release', 'number-observations'])

    df = market.fetch_market(market_data_request)

    print(df)

    # now just download the event day
    market_data_request = MarketDataRequest(
                    start_date = "year",
                    category = "events",
                    data_source = 'bloomberg',              # use Bloomberg as data source
                    tickers = ['NFP'],
                    fields = ['release-date-time-full'],                # which fields to download
                    vendor_tickers = ['NFP TCH Index'],     # ticker (Bloomberg)
                    vendor_fields = ['ECO_FUTURE_RELEASE_DATE_LIST'])     # which Bloomberg fields to download

    df = market.fetch_market(market_data_request)

    print(df)


__author__ = 'saeedamen' # Saeed Amen

#
# Copyright 2016 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

"""
TickerFactory

Creating lists of tickers from a shortcuts.

"""

import pandas
import numpy

from findatapy.util.loggermanager import LoggerManager

class TickerFactory(object):

    def create_ticker(self, csv_file, out_csv_file):
        # reader = csv.DictReader(open(csv_file))

        data_frame = pandas.read_csv(csv_file, dtype=object)

        rows = 100000
        # category	source	freq	ticker	cut	fields	sourceticker

        data_frame_out = pandas.DataFrame(index=numpy.arange(0, rows),
                                          columns=('category', 'source', 'freq', 'ticker', 'cut', 'fields', 'sourceticker'))
        i = 0

        for category_source_freq_fields in data_frame['category.source.freq.fields']:
            if isinstance(category_source_freq_fields, str):
                spl = category_source_freq_fields.split('.')
                category = spl[0]
                source = spl[1]
                freq = spl[2]
                fields = spl[3]

                for cut_postfix in data_frame['cut.postfix']:
                    if isinstance(cut_postfix, str):
                        spl1 = cut_postfix.split('.')
                        cut = spl1[0]
                        postfix = spl1[1]
                        for ticker, st in zip(data_frame['ticker'], data_frame['sourceticker']):
                            if isinstance(ticker, str):
                                if 'midfix' in data_frame.columns:
                                    for midfix in data_frame['midfix']:
                                        if isinstance(midfix, str):
                                            for postmidfix in data_frame['postmidfix']:
                                                if isinstance(postmidfix, str):
                                                    ticker_ext = ticker + midfix + postmidfix
                                                    sourceticker = st + midfix + postmidfix + ' ' + postfix
                                                    data_frame_out.loc[i] = [category, source, freq, ticker_ext,
                                                                             cut, fields, sourceticker]

                                                    i = i + 1
                                else:
                                    for postmidfix in data_frame['postmidfix']:
                                        if isinstance(postmidfix, str):
                                            ticker_ext = ticker + postmidfix
                                            sourceticker = st + postmidfix + ' ' + postfix
                                            data_frame_out.loc[i] = [category, source, freq, ticker_ext, cut, fields,
                                                                     sourceticker]

                                            i = i + 1


        data_frame_out = data_frame_out[0:i]

        data_frame_out.to_csv(out_csv_file)

        # for line in reader:
        #     category = line["category"]
        #     source = line["source"]
        #     freq = line["freq"]
        #     ticker = line["ticker"]
        #     cut = line["cut"]
        #     sourceticker = line["sourceticker"]


if __name__ == '__main__':

    logger = LoggerManager.getLogger(__name__)

    tf = TickerFactory()

    root = 'E:/cuemacro/yen/conf'
    # root = 'E:/cuemacro/findatapy/findatapy/conf'

    csv_file = root + '/fx_vol_tickers_maker.csv'
    out_csv_file = root + '/fx_vol_tickers.csv'

    tf.create_ticker(csv_file, out_csv_file)

    csv_file = root + '/fx_forwards_tickers_maker.csv'
    out_csv_file = root + '/fx_forwards_tickers.csv'

    tf.create_ticker(csv_file, out_csv_file)



from __future__ import print_function

__author__ = 'saeedamen'  # Saeed Amen / saeed@cuemacro.com

#
# Copyright 2017 Cuemacro Ltd. - http//www.cuemacro.com / @cuemacro
#
# See the License for the specific language governing permissions and limitations under the License.
#

from findatapy.util.singleton import Singleton

import threading

class Mediator(object):
    """Mediator acts as a source for static/one off objects such as the VolatileCache, TCAMarketTradeLoader,
    TCATickerLoader etc.  Also allows users to create their own subclasses of these and distribute universally to the
    project, without having to worry about importing the appropriate classes/implementations.

    """
    __metaclass__ = Singleton

    _data_constants = None
    _data_constants_lock = threading.Lock()

    def __init__(self, *args, **kwargs):
        pass

    @staticmethod
    def get_data_constants():
        with Mediator._data_constants_lock():

            from findatapy.util.dataconstants import DataConstants

            Mediator._data_constants = DataConstants()

        return Mediator._data_constants





import keyring
import os

class KeyringFetch(object):
    """Functions for converting between vendor tickers and findatapy tickers (and vice-versa).

    """

    def __init__(self, *args, **kwargs):
        pass

    @staticmethod
    def key_store(service_name):
        key = None

        # This will fail on some cloud notebook platforms so put in try/except loop
        try:
            key = keyring.get_password(service_name, os.getlogin())
        except:
            pass

        return key